pro read_product_sds,datavalue,allvalidrange, fileId, varname,fail, $
                  scaleNo=scaleNo, offsetNo=offsetNo, fillNo=fillNo
;+
;.......................................................................
; $Id$
;.......................................................................
; NAME: ReadSDS
;	
;
; PURPOSE:
;        given file id and a variable name, retrieve "calibrated" data 
;        values.
;	
;
; CATEGORY:
;        HDF data reader
;
;
; CALLING SEQUENCE:
;	Result = ReadSDS(fileId, varname)
;
; INPUTS:
;	fileId:	HDF file id number
;       varname: variable name
;
; OPTIONAL INPUTS:
;       scaleNo:  This parameter does not have scale factor attribute;
;                 default assumes that it has.
;       offsetNo: This parameter does not have offset attribute; 
;                 default assumes that it has.
;       fillNo:   This parameter does not have fill value attribute;
;                 default assumes that it has.
;
;
; OUTPUTS:
;       calibrated data in proper physical units
;
;
; USAGE EXAMPLES:
;
; fileId = HDF_SD_START("sample.hdf", /READ)
; variable = ReadSDS(fileId, 'Cloud_Mask')
; HDF_SD_END, fileId
;
; SIDE EFFECTS:
;       None.
;.......................................................................
; REVISION HISTORY:
;       3/7/00	MAG	Updates with proper MODIS names and additional
;                       code to take into account incorrectly set FillValues
;                       in the L-1B data itself. 
;.......................................................................
; PROGRAMMER:  
;              Mark A Gray (gray@climate.gsfc.nasa.gov
;
;              Jason Li 
;              SM&A Inc | Climate and Radiation Branch
;              NASA Goddard Space Flight Center
;              Greenbelt, Maryland, U.S.A.
;.......................................................................
;-

fail=0

;... read SDS data:

index = HDF_SD_NAMETOINDEX(fileId, varname)
sdsid = HDF_SD_SELECT(fileId, index)
HDF_SD_GETDATA, sdsid, dataValue
HDF_SD_ENDACCESS, sdsid

;... read attributes:

scale_sds_name= 'scale_factor'
IF NOT Keyword_Set(scaleNo) THEN BEGIN
   scaleId = HDF_SD_ATTRFIND(sdsid,scale_sds_name)
   IF scaleId NE -1 THEN BEGIN
      HDF_SD_ATTRINFO, sdsid, scaleId, DATA=scale_factor
   ENDIF
ENDIF

offset_sds_name= 'add_offset'
IF NOT Keyword_Set(offsetNo) THEN BEGIN
   offsetId = HDF_SD_ATTRFIND(sdsid,offset_sds_name)
   IF offsetId NE -1 THEN BEGIN
      HDF_SD_ATTRINFO, sdsid, offsetId, DATA=offset
   ENDIF
ENDIF

validrange_sds_name= 'valid_range'
IF NOT Keyword_Set(validrangeNo) THEN BEGIN
   validrangeId = HDF_SD_ATTRFIND(sdsid,validrange_sds_name)
   IF validrangeId NE -1 THEN BEGIN
      HDF_SD_ATTRINFO, sdsid, validrangeId, DATA=validrange
   ENDIF
ENDIF
allvalidrange = validrange

fillvalue_sds_name= '_FillValue'
IF NOT Keyword_Set(fillNo) THEN BEGIN
   fillId = HDF_SD_ATTRFIND(sdsid,fillvalue_sds_name)
   IF fillId NE -1 THEN BEGIN
      HDF_SD_ATTRINFO, sdsid, fillId, DATA=fillValue
   ENDIF
   fillvalue=fillvalue[0]
ENDIF

;... calibrate data by applying scale factor and offset if necessary:

arraysizetemp=size(dataValue)
;datavalue=float(datavalue)


;3/5/00 : Apparently, FillValue is sometimes -1 now in L-1B files
;         we set all fillvalues points to -1 here and then proceed

;fudge for dodgy fillval in level 1b files, sometimes a fillvalue of
;65535 is set to 65528
;set all filled values to -1 and set fill value to -1 for the remainder 
;of processing

;if fillvalue gt 65000 then begin
;  fill_index=where(DataValue gt fillValue*.99,count)
;  if count gt 0 then DataValue(fill_index)=-1
;  fillvalue=-1
;endif
;
if not keyword_set(scaleNo) then begin
case 1 of
   max(DataValue) eq -1 and min(DataValue) eq -1: begin
      ;data is a probably a vis channel viewed at nighttime 
      ;let's get the hell out here! eek!
      print,'Cannot find any useful data in the sds ',varname
      fail= 1
   end
   arraysizetemp(0) eq 2: begin
       allvalidrange=fltarr(2)
      index = Where(dataValue ne fillValue[0], count)
      if count eq 0 then begin
          index=where(datavalue le 0.,count)
          datavalue[index]=0.
      endif 
      dataValue[index] = scale_factor[0] * (dataValue[index]-offset[0])
      allvalidrange[0:1]=scale_factor[0] * (validrange[0:1]-offset[0])

      dum=where(datavalue lt 0., dumcount)
      if dumcount gt 0 then  datavalue[dum]=0.


   end
   arraysizetemp(0) eq 3: begin

      allvalidrange=fltarr(2,arraysizetemp[3])
      for band =0, arraysizetemp[3]-1 do begin
         bandDataValue =dataValue[*,*,band] 
         allvalidrange[*,band]= scale_factor[band] * (validrange-offset[band])


         index = Where(bandDataValue ne fillValue, count)
         bandDataValue[index] = scale_factor[band] * (bandDataValue[index]-offset[band])
         index=where(bandDataValue eq  fillValue, count)
         if count gt 0 then bandDataValue[index] = 0.

         dataValue[*,*,band]=bandDataValue
      endfor
   end
else:print,'Oh my, in big trouble here.  your crash is being served'

endcase
endif
end
