function l2qa_bitextracter, MOD06FileName

; IDL routine
;
; l2qa_BitExtracter.pro v 1.0
;
; l2qa_BitExtracter.pro extracts the individual QA flags from the full byte
;   array 1km QA and places the results into an output array.  This routine
;   extracts QA from MOD06, but can be used as an example/template for 
;   other QA or bit flags extracted from byte arrays.
;
; Input: 
;    MOD06FileName	String	   : Full path and name of the ecosystem ancillary file
; 
; Output:
;    QABitFlags 	Byte Array  : The individual results of the QA flags,
;				     written as follows:
;			L2QAFlags(*,*,0:19)
;			L2QAFlags(*,*, 0) = Optical Thickness General QA
;			L2QAFlags(*,*, 1) = Optical Thickness Confidence QA
;			L2QAFlags(*,*, 2) = Optical Thickness out-of-bounds
;			L2QAFlags(*,*, 3) = Effective Radius General QA
;			L2QAFlags(*,*, 4) = Effective Radius Confidence QA
;			L2QAFlags(*,*, 5) = Liquid Water Path General QA
;			L2QAFlags(*,*, 6) = Liquid Water Path Confidence QA
;			L2QAFlags(*,*, 7) = Cloud Phase Determine (SWIR)
;			L2QAFlags(*,*, 8) = Cloud Phase used in retrieval proc. path
;	                L2QAFlags(*,*, 9) = Retrieval Outcome
;			L2QAFlags(*,*,10) = Rayleigh Correction
;			L2QAFlags(*,*,11) = Water Vapor Correction
;			L2QAFlags(*,*,12) = Band Used for Optical Thickness Retrieval
;			L2QAFlags(*,*,13) = Total Precipitable Water
;			L2QAFlags(*,*,14) = Cloud Top Height
;			L2QAFlags(*,*,15) = Temperature Profile
;			L2QAFlags(*,*,16) = Surface Temperature Over Land
;			L2QAFlags(*,*,17) = Surface Temperature Over Ocean
;			L2QAFlags(*,*,18) = BRDF/Albedo
;			L2QAFlags(*,*,19) = Ozone Profile
;
; Required routines:
;    		ReadSDS
;
; Developer
;  v 1.0    03/20/2001
;       	Eric Moody moody@climate.gsfc.nasa.gov
;			Jason Li 
;         Emergent IT
;           MODIS - Atmosphere Group
;


;Read in the QA, 1km:
MOD06FileID = HDF_SD_START(MOD06FileName,/READ)
L2QA = ReadSDS(MOD06FileID,'Quality_Assurance_1km',/scaleNo,/offsetNo,/fillNo)
HDF_SD_END, MOD06FileID


;Determine the size of the Cloud Mask array:
dims = size(L2QA,/dimensions)
xdim = dims(1)
ydim = dims(2)
nbands = dims(0)


;Transpose the array from band,x,y to x,y,band:
L2QA = TRANSPOSE(L2QA, [1, 2, 0])


;Create the integer L2QAFlags output array:
L2QAFlags = IntArr(xdim,ydim,20)

L2QAFlags2 = IntArr(xdim,ydim)

;Extract Flags from the Zeroeth Byte:

;Extract the Optical Thickness General QA:
L2QAFlags[*,*, 0] =       (L2QA[*,*,0] AND   1B)


;Extract the Optical Thickness Confidence QA:
L2QAFlags[*,*, 1] = ISHFT((L2QA[*,*,0] AND   6B), -1)


;Extract the Optical Thickness out-of-bounds:
L2QAFlags[*,*, 2] = ISHFT((L2QA[*,*,0] AND  24B), -3)


;Extract the Effective Radius General QA:
L2QAFlags[*,*, 3] = ISHFT((L2QA[*,*,0] AND  32B), -5)


;Extract the Effective Radius Confidence QA:
L2QAFlags[*,*, 4] = ISHFT((L2QA[*,*,0] AND 192B), -6)



;Extract Flags from the First Byte:

;Extract the Liquid Water Path General QA:
L2QAFlags[*,*, 5] =       (L2QA[*,*,1] AND   1B)


;Extract the Liquid Water Path Confidence QA:
L2QAFlags[*,*, 6] = ISHFT((L2QA[*,*,1] AND   6B), -1)


;Extract the Cloud Phase Determination (SWIR):
L2QAFlags[*,*, 7] = ISHFT((L2QA[*,*,1] AND  56B), -3)



;Extract Flags from the Second Byte:

;Extract the Cloud Phase used in retrieval processing path:
L2QAFlags[*,*, 8] =       (L2QA[*,*,2] AND   7B)

;Extract the Retrieval Outcome:
L2QAFlags[*,*, 9] = ISHFT((L2QA[*,*,2] AND   8B), -3)

;Extract the Rayleigh Correction:
L2QAFlags[*,*,10] = ISHFT((L2QA[*,*,2] AND  16B), -4)


;Extract the Water Vapor Correction:
L2QAFlags[*,*,11] = ISHFT((L2QA[*,*,2] AND  32B), -5)


;Extract the Band Used for Optical Thickness Retrieval:
L2QAFlags[*,*,12] = ISHFT((L2QA[*,*,2] AND 192B), -6)



;Extract Flags from the Third Byte:

;Extract the Total Precipital Water:
L2QAFlags[*,*,13] =       (L2QA[*,*,3] AND   3B)


;Extract the Cloud Top Height:
L2QAFlags[*,*,14] = ISHFT((L2QA[*,*,3] AND  12B), -2)


;Extract the Temperature Profile:
L2QAFlags[*,*,15] = ISHFT((L2QA[*,*,3] AND  48B), -4)


;Extract the Surface Temperature Over Land:
L2QAFlags[*,*,16] = ISHFT((L2QA[*,*,3] AND 192B), -6)



;Extract Flags from the Fourth Byte:

;Extract the Surface Temperature Over Ocean:
L2QAFlags[*,*,17] =       (L2QA[*,*,4] AND   3B)


;Extract the BRDF/Albedo:
L2QAFlags[*,*,18] = ISHFT((L2QA[*,*,4] AND  12B), -2)


;Extract the Ozone Profile:
L2QAFlags[*,*,19] = ISHFT((L2QA[*,*,4] AND  48B), -4)


;Change the Integer L2QAFlags array into a byte array:
L2QAFlags = Byte(L2QAFlags)


; Return the Extracted L2 QA Flags:
return, L2QAFlags

end
